indexing

	description: "Paul Graham's hash of tokens and message count."

	author: "Berend de Boer"
	copyright: "(c) 2003, Eiffel Forum License v2, see forum.txt"


class

	TOKEN_TABLE


inherit

	ANY

	KL_SHARED_STANDARD_FILES
		export
			{NONE} all
		end

	STRING_ROUTINES
		export
			{NONE} all
		end

	EXCEPTIONS
		export
			{NONE} all
		end


creation

	make,
	make_from_file


feature {NONE} -- Initialization

	make (a_message_count: INTEGER; a_tokens: DS_HASH_TABLE [INTEGER, STRING]) is
			-- Initialize from given message count and token has table.
		require
			message_count_not_negative: a_message_count >= 0
			tokens_not_void: a_tokens /= Void
		do
			message_count := a_message_count
			tokens := a_tokens
		ensure
			message_count_set: a_message_count = message_count
		end

	make_default is
			-- Create an empty token file.
		do
			create tokens.make_default
		end

	make_from_file (a_file_name: STRING) is
			-- Initialize `message_count' and `tokens' from a file.
			-- @@BdB: limited error checking!
		require
			a_file_name_not_empty: a_file_name /= Void and then not a_file_name.is_empty
		local
			initial_size: INTEGER
			file: KL_TEXT_INPUT_FILE
			occurrences: INTEGER
			token: STRING
			ar: ARRAY [STRING]
		do
			create file.make (a_file_name)
			file.open_read
			if file.is_open_read then
				file.read_line
				message_count := file.last_string.to_integer
				-- I'm still not sure if the number passed to the hash
				-- table is actually the number of expected items, or that
				-- it influences the hashing as well.
				file.read_line
				initial_size := file.last_string.to_integer
				create tokens.make (initial_size)
				from
					file.read_line
				until
					file.end_of_input
				loop
					file.last_string.left_adjust
					file.last_string.right_adjust
					ar := split_on (file.last_string, ' ')
					if not ar.item (ar.lower).is_integer then
						std.error.put_string ("Non-integer occurrence encountered.%N")
						die (1)
					end
					occurrences := ar.item (ar.lower).to_integer
					token := ar.item (ar.upper)
					if token.is_empty then
						std.error.put_string ("Empty token encountered.%N")
						die (1)
					end
					tokens.put (occurrences, token)
					file.read_line
				end
			else
				std.error.put_string ("File ")
				std.error.put_string (a_file_name)
				std.error.put_string (" cannot be opened for read.%N")
				die (1)
			end
		end


feature -- Access

	message_count: INTEGER

	tokens: DS_HASH_TABLE [INTEGER, STRING]


feature -- Change

	wipe_out is
		-- Erase all tokens.
		do
			tokens.wipe_out
			message_count := 0
		ensure
			message_count_is_zero: message_count = 0
			tokens_empty: tokens.is_empty
		end


invariant

	message_count_not_negative: message_count >= 0

	tokens_not_void: tokens /= Void

end
