indexing

	description: "Parses an EMC configuration file and sets the given configuration object."

	author: "Berend de Boer"


class

	CONFIGURATION_PARSER


inherit

	ANY

	EPX_CURRENT_PROCESS
		export
			{NONE} all
		end

	ST_FORMATTING_ROUTINES
		export
			{NONE} all
		end

	XM_CALLBACKS_NULL
		rename
			make as make_null
		export
			{NONE} all
		end

	KL_IMPORTED_STRING_ROUTINES
		export
			{NONE} all
		end


creation

	make


feature -- Initialization

	make is
			-- Initialize parser.
		local
			xp: XM_EIFFEL_PARSER
		do
			make_null
			create xp.make
			xp.set_string_mode_unicode
			create vp.make (xp)
			vp.set_callbacks (Current)
			create schema
			schema.build_schema (vp)
			schema.classify.set_on_start_tag (agent on_start_classify)
			schema.classify.set_on_end_tag (agent on_end_classify)
			schema.host.set_on_end_tag (agent on_end_host)
			schema.login_name.set_on_end_tag (agent on_end_login_name)
			schema.mailbox.get_attribute ("highest-sequence").set_on_content_save
			schema.mailbox.get_attribute ("max-message-size").set_on_content_save
			schema.mailbox.get_attribute ("max-messages").set_on_content_save
			schema.mailbox.get_attribute ("lowest-sequence").set_on_content_save
			schema.mailbox.get_attribute ("percentage").set_on_content_save
			schema.mailbox.set_on_start_tag (agent on_start_mailbox)
			schema.mailbox.set_on_end_tag (agent on_end_mailbox)
			schema.move_nonspam_to.set_on_end_tag (agent on_end_move_nonspam_to)
			schema.move_spam_to.set_on_end_tag (agent on_end_move_spam_to)
			schema.new_word_spaminess.set_on_end_tag (agent on_end_new_word_spaminess)
			schema.nonspam.get_attribute ("max-message-size").set_on_content_save
			schema.nonspam.get_attribute ("max-messages").set_on_content_save
			schema.nonspam.set_on_start_tag_finish (agent on_start_nonspam_finish)
			schema.nonspam.set_on_end_tag (agent on_end_nonspam)
			schema.password.set_on_end_tag (agent on_end_password)
			schema.port.set_on_end_tag (agent on_end_port)
			schema.scan.get_attribute ("minimum-occurrences").set_on_content_save
			schema.scan.get_attribute ("minimum-occurrences").data.set_default_data (1)
			schema.scan.set_on_start_tag (agent on_start_scan)
			schema.scan.set_on_start_tag_finish (agent on_start_scan_finish)
			schema.secure.set_on_end_tag (agent on_end_secure)
			schema.spam.get_attribute ("max-message-size").set_on_content_save
			schema.spam.get_attribute ("max-messages").set_on_content_save
			schema.spam.set_on_start_tag_finish (agent on_start_spam_finish)
			schema.spam.set_on_end_tag (agent on_end_spam)
			schema.strength.set_on_end_tag (agent on_end_strength)
			schema.threshold.set_on_end_tag (agent on_end_threshold)
			schema.lower_minimum_deviation.set_on_end_tag (agent on_end_lower_minimum_deviation)
			schema.upper_minimum_deviation.set_on_end_tag (agent on_end_upper_minimum_deviation)
		end


feature -- Access

	is_correct: BOOLEAN is
			-- Was the last handled dispatch ok?
		do
			Result := vp.is_correct
		end

	last_error: STRING is
			-- An error message, if any.
		require
			there_is_an_error: not is_correct
		do
			Result := vp.last_error_extended_description
		ensure
			result_not_void: Result /= Void
		end


feature -- Commands

	parse_from_stream (a_stream: KI_CHARACTER_INPUT_STREAM; a_configuration: CONFIGURATION) is
			-- Parse stream. Use `is_correct' to determine if parsing was
			-- successful.
		require
			a_stream_not_void: a_stream /= Void
			is_open_read: a_stream.is_open_read
			configuration_not_void: a_configuration /= Void
		do
			configuration := a_configuration
			in_classify := False
			in_spam := False
			in_nonspam := False
			vp.parse_from_stream (a_stream)
		end


feature {NONE} -- Callbacks to specific elements

	on_end_classify (element: XM_DEFINED_ELEMENT; a_content: STRING) is
		do
			in_classify := False
		end

	on_end_host (element: XM_DEFINED_ELEMENT; a_content: STRING) is
		do
			configuration.set_host (a_content.twin)
		end

	on_end_login_name (element: XM_DEFINED_ELEMENT; a_content: STRING) is
		do
			configuration.set_login_name (a_content.twin)
		end

	on_start_mailbox (element: XM_DEFINED_ELEMENT) is
		do
			element.reset_to_default_data
		end

	on_end_mailbox	(element: XM_DEFINED_ELEMENT; a_content: STRING) is
		local
			mailbox: MAILBOX
		do
			create mailbox.make (
				a_content.twin,
				element.get_attribute ("max-message-size").data.as_integer,
				element.get_attribute ("max-messages").data.as_integer,
				element.get_attribute ("percentage").data.as_integer,
				element.get_attribute ("lowest-sequence").data.as_integer,
				element.get_attribute ("highest-sequence").data.as_integer)
			if in_spam then
				if not configuration.has_spam_mailbox (mailbox.name) then
					configuration.spam_mailboxes.put_last (mailbox)
				else
					fd_stderr.put_string (format ("Mailbox $s is listed twice inside the <spam> element. You should remove the duplicate.%N", <<mailbox.name>>))
				end
			elseif in_nonspam then
				if not configuration.has_nonspam_mailbox (mailbox.name) then
					configuration.nonspam_mailboxes.put_last (mailbox)
				else
					fd_stderr.put_string (format ("Mailbox $s is listed twice inside the <nonspam> element. You should remove the duplicate.%N", <<mailbox.name>>))
				end
			elseif in_classify then
				configuration.set_scan_mailbox (mailbox)
			end
		end

	on_end_move_nonspam_to (element: XM_DEFINED_ELEMENT; a_content: STRING) is
		do
			configuration.set_nonspam_mailbox (a_content.twin)
		end

	on_end_move_spam_to (element: XM_DEFINED_ELEMENT; a_content: STRING) is
		do
			configuration.set_spam_mailbox (a_content.twin)
		end

	on_end_new_word_spaminess (element: XM_DEFINED_ELEMENT; a_content: STRING) is
		do
			configuration.set_new_word_spaminess (a_content.to_double)
		end

	on_end_nonspam (element: XM_DEFINED_ELEMENT; a_content: STRING) is
		do
			in_nonspam := False
			schema.mailbox.get_attribute ("max-message-size").data.set_default_data (0)
			schema.mailbox.get_attribute ("max-messages").data.set_default_data (0)
		end

	on_end_password (element: XM_DEFINED_ELEMENT; a_content: STRING) is
		do
			configuration.set_password (a_content.twin)
		end

	on_end_port (element: XM_DEFINED_ELEMENT; a_content: STRING) is
		do
			configuration.set_port (a_content.to_integer)
		end

	on_end_secure (element: XM_DEFINED_ELEMENT; a_content: STRING) is
		local
			s: STRING
		do
			s := a_content.as_lower
			if
				(s.is_integer and then s.to_integer /= 0) or else
				(STRING_.same_string (s, once_true))
			then
				configuration.set_secure (True)
			else
				configuration.set_secure (False)
			end
		end

	on_end_spam (element: XM_DEFINED_ELEMENT; a_content: STRING) is
		do
			in_spam := False
			schema.mailbox.get_attribute ("max-message-size").data.set_default_data (0)
			schema.mailbox.get_attribute ("max-messages").data.set_default_data (0)
		end

	on_end_strength (element: XM_DEFINED_ELEMENT; a_content: STRING) is
		do
			configuration.set_strength (a_content.to_double)
		end

	on_end_threshold (element: XM_DEFINED_ELEMENT; a_content: STRING) is
		do
			configuration.set_threshold (a_content.to_double)
		end

	on_end_lower_minimum_deviation (element: XM_DEFINED_ELEMENT; a_content: STRING) is
		do
			configuration.set_lower_minimum_deviation (a_content.to_double)
		end

	on_end_upper_minimum_deviation (element: XM_DEFINED_ELEMENT; a_content: STRING) is
		do
			configuration.set_upper_minimum_deviation (a_content.to_double)
		end

	on_start_classify (element: XM_DEFINED_ELEMENT) is
		do
			in_classify := True
		end

	on_start_nonspam_finish (element: XM_DEFINED_ELEMENT) is
		do
			in_nonspam := True
			schema.mailbox.get_attribute ("max-message-size").data.set_default_data (element.get_attribute ("max-message-size").data.as_integer)
			schema.mailbox.get_attribute ("max-messages").data.set_default_data (element.get_attribute ("max-messages").data.as_integer)
		end

	on_start_scan (element: XM_DEFINED_ELEMENT) is
		do
			element.reset_to_default_data
		end

	on_start_scan_finish (element: XM_DEFINED_ELEMENT) is
		do
			configuration.set_minimum_occurrences (element.get_attribute ("minimum-occurrences").data.as_integer)
		end

	on_start_spam_finish (element: XM_DEFINED_ELEMENT) is
		do
			in_spam := True
			schema.mailbox.get_attribute ("max-message-size").data.set_default_data (element.get_attribute ("max-message-size").data.as_integer)
			schema.mailbox.get_attribute ("max-messages").data.set_default_data (element.get_attribute ("max-messages").data.as_integer)
		end


feature {NONE} -- Implementation

	configuration: CONFIGURATION
			-- The configuration that is currently parsed.

	schema: EMC_SCHEMA
			-- Schema used to validate configuration.

	vp: XM_VALIDATING_PARSER


feature {NONE} -- Callback state

	in_classify: BOOLEAN
			-- Inside the <classify> tag.

	in_spam: BOOLEAN
			-- Inside the <spam> tag.

	in_nonspam: BOOLEAN
			-- Inside the <nonspam> tag.


feature {NONE} -- Once constants

	once_true: STRING is "true"


invariant

	vp_not_void: vp /= Void

end
