indexing

	description: "EMC configuration contents."

	author: "Berend de Boer"


class

	MAILBOX


inherit

	ANY

	KL_SHARED_PLATFORM
		export
			{NONE} all
		end


creation

	make


feature {NONE} -- Initialization

	make (
			a_name: STRING;
			a_max_message_size: INTEGER;
			a_max_messages: INTEGER;
			a_percentage: INTEGER;
			a_lowest_sequence,
			a_highest_sequence: INTEGER) is
			-- Initialize a mailbox.
		require
			name_not_empty: a_name /= Void and then not a_name.is_empty
			max_message_size_not_negative: a_max_message_size >= 0
			max_messages_not_negative: a_max_messages >= 0
			percentage_not_negative: a_percentage >= 0
			lowest_sequence_not_negative: a_lowest_sequence >= 0
			highest_sequence_not_negative: a_highest_sequence >= 0
			lowest_sequence_not_larger_than_highest_sequence:
				highest_sequence /= 0 implies
					lowest_sequence <= highest_sequence
		do
			name := a_name
			if a_max_message_size = 0 then
				max_message_size := Platform.Maximum_integer
			else
				max_message_size := a_max_message_size
			end
			max_messages := a_max_messages
			percentage := a_percentage
			if a_lowest_sequence /= 0 then
				lowest_sequence := a_lowest_sequence
			else
				lowest_sequence := 1
			end
			highest_sequence := a_highest_sequence
		end


feature -- Access

	name: STRING
			-- Mailbox name.

	max_message_size: INTEGER
			-- Maximum size of message to be read.
			-- Useful for a slow connection. Multiple MB's large messages
			-- are just skipped.

	max_messages: INTEGER
			-- Maximum number of messages to process for this mailbox if
			-- not zero.

	percentage: INTEGER
			-- Percentage of messages in mailbox to process if not zero.

	lowest_sequence: INTEGER
			-- Lowest message sequence number in this mailbox to read.

	highest_sequence: INTEGER
			-- Highest message sequence number in this mailbox to read,
			-- if not zero.


invariant

	name_not_empty: name /= Void and then not name.is_empty
	max_message_size_positive: max_message_size >= 1
	max_messages_not_negative: max_messages >= 0
	percentage_not_negative: percentage >= 0
	lowest_sequence_positive: lowest_sequence >= 1
	highest_sequence_not_negative: highest_sequence >= 0
	lowest_sequence_not_larger_than_highest_sequence:
		highest_sequence /= 0 implies
			lowest_sequence <= highest_sequence

end
