indexing

	description: "Accepts tokens and hashes them."

	author: "Berend de Boer"


class

	TOKEN_HASHER


inherit

	PW_WRITER [STRING]


creation

	make


feature -- Initialization

	make is
		do
			create hash_table.make (initial_size)
		end


feature -- Access

	hash_table: DS_HASH_TABLE [INTEGER, STRING]


feature -- Writing

	is_valid_item (item: STRING): BOOLEAN is
			-- Does `item' not contain any white-space?
		do
			Result :=
				item /= Void and then
				not item.is_empty and then
				not item.has (' ') and then
				not item.has ('%T')
		end

	put (token: STRING) is
			-- Hash `token'. We assume `token' does not change behind our back.
		local
			key: STRING
			c: INTEGER
		do
				check
					valid: is_valid_item (token)
				end
			hash_table.search (token)
			if hash_table.found then
				hash_table.replace_found_item (hash_table.found_item + 1)
			else
				c := hash_table.count
				-- Make sure `key' is not aliased
				-- @@BdB: should not be needed, removed for performance
				--create key.make_from_string (token)
				key := token
				hash_table.force (1, key)
					check
						one_token_added: hash_table.count = c + 1
					end
			end
		ensure then
			hashed: hash_table.has (token)
		end


feature {NONE} -- Implementation

	initial_size: INTEGER is 16384


invariant

	hash_table_not_void: hash_table /= Void
	tokens_have_no_white_space: True
			-- for_each t in hash_table.keys it_holds not t.has_whitespace

end
