indexing

	description: "Class that accepts text streams, tokenizes them, and builds the hash table."

	author: "Berend de Boer"


class

	TOKENIZER


creation

	make


feature {NONE} -- Initialization

	make is
		do
			make_pipe
		end


feature -- Access

	count: INTEGER
			-- Number of tokenized texts.

	hash_table: DS_HASH_TABLE [INTEGER, STRING] is
			-- The tokens.
		do
			Result := token_hasher.hash_table
		ensure
			hash_table_not_void: hash_table /= Void
		end


feature -- Change

	wipe_out is
		-- Erase all tokens.
		do
			hash_table.wipe_out
			count := 0
		ensure
			count_is_zero: count = 0
			hash_table_empty: hash_table.is_empty
		end


feature -- Action

	tokenize (text: STRING) is
			-- Tokenize `text', but do not increment `count'.
		require
			text_not_empty: text /= Void and then not text.is_empty
		do
			character_source.connect (text)
			pump.run
			character_source.disconnect
		ensure
			count_unchanged: count = old count
		end

	tokenize_special (a_token: STRING) is
			-- Add token `a_token'. Do not increment `count'.
			-- `a_token' will be tokenized with a special character that
			-- cannot not appear in tokenized text.
			-- This is a trick, so the format of the probabilities.txt
			-- does not need to change. It could become a separate
			-- section perhaps.
		require
			token_not_empty: a_token /= Void and then not a_token.is_empty
			no_blanks: not a_token.has (' ') and not a_token.has ('%T')
		do
			token_hasher.put (to_special (a_token))
		ensure
			count_unchanged: count = old count
			token_hasher_has_token: token_hasher.hash_table.has (to_special (a_token))
		end

	start_new_text is
			-- Increment `count', i.e. the number of messages parsed.
		do
			count := count + 1
		ensure
			one_text_more_counted: count = old count + 1
		end


feature {NONE} -- Implementation

	to_special (a_token: STRING): STRING is
		require
			token_not_empty: a_token /= Void and then not a_token.is_empty
		do
			create Result.make (a_token.count + 1)
			Result.append_character ('$')
			Result.append_string (a_token)
		ensure
			special_not_empty: Result /= Void and then not Result.is_empty
		end


feature {NONE} -- Pipe

	make_pipe is
		local
			token_writer: TOKEN_WRITER
		do
			-- reader, generates stream of tokens in the form of STRINGs
			create {PW_STRING_CHARACTER_SOURCE} character_source.make

			-- writer, update `hash_table'
			create token_writer.make
			create token_hasher.make
			token_writer.connect (token_hasher)

			-- pump
			create pump.make
			pump.connect (character_source, token_writer)

		end

	character_source: PW_SOURCE_READER [CHARACTER]

	token_hasher: TOKEN_HASHER

	pump: PW_PIPE_PUMP [CHARACTER]
			-- The pump used in `tokenize'.


end
